#!/usr/bin/python

import os

import enchant
from enchant.checker import SpellChecker
from mercurial import commands, extensions, util

def get_checker(ui):
	locale = ui.config('spellcheck', 'locale', 'en_US')
	pdictfile = ui.config('spellcheck', 'dictfile', '.dict-validwords')
	
	checker = SpellChecker(locale)
	
	if os.path.exists(pdictfile):
		for line in open(pdictfile):
			checker.add(line.strip())
			
	return checker

def hook(ui, repo, **kwargs):
	"""This hook fails if the commit message for the `node` parameter has spelling
	errors.
	"""
	
	if not 'node' in kwargs:
		raise util.Abort('You must use the spellcheck hook in a context that supplies a `node` parameter (e.g. pretxncommit)')
		
	checker = get_checker(ui)
	checker.set_text(repo[kwargs['node']].description())
	
	errors = 0
	
	for err in checker:
		errors += 1
		err.replace('>%s<' % err.word)
	
	if errors:	
		raise util.Abort('spelling error: %s' % checker.get_text())

def spellchecked(orig, ui, repo, *pats, **opts):
	locale = ui.config('spellcheck', 'locale', 'en_US')
	pdictfile = ui.config('spellcheck', 'dictfile', '.dict-validwords')
	
	msg = opts['message']
	
	checker = get_checker(ui)
	
	checker.set_text(msg)
	leftover = None
	
	for err in checker:
		w = err.word
		
		if w == leftover:
			continue

		suggestions = err.suggest()
		
		keys = ['&%s' % x for x in 'abcdef'.upper()][:len(suggestions)]
		ui.warn(checker.get_text().replace(w, '>%s<' % w), '\n\n')
		
		for letter, s in zip('abcdef'.upper(), suggestions):
			ui.write('  %s. %s\n' % (letter, checker.get_text().replace(w, '>%s<' % s)))
		
		ui.write('\n')	
		ui.write('  X. Custom replacement\n')
		ui.write('  Q. Cancel commit\n')
		ui.write('  +. Add "%s" to personal dictionary\n' % w)
		
		ri = ui.promptchoice('\n[%sxq+]:' % ''.join(k.replace('&', '').lower() for k in keys), keys + ['&X', '&Q', '&+'], 0)

		if ri == len(keys):
			checked = False
			while not checked:
				repl = ui.prompt('Replacement text [%s]:' % w, default=w)
				if repl == w:
					break
				checked = checker.check(repl)
				if not checked:
					ui.warn('  Spelling error in correction text!\n')
		elif ri > len(keys):
			if ri == len(keys) + 1:
				raise util.Abort('commit message failed spellcheck.')
			elif ri == len(keys) +2:
				repl = w
				checker.add(w)
				f = open(pdictfile, 'a')
				print >>f, w
				f.close()
		else:
			repl = suggestions[ri]
		
		if len(w) > len(repl):
			leftover = w[len(repl):]
			
		err.replace(repl)

	opts['message'] = checker.get_text()
	return orig(ui, repo, *pats, **opts)
	
def uisetup(ui):
	if not hasattr(extensions, 'wrapcommand'):
		return
	extensions.wrapcommand(commands.table, 'commit', spellchecked)



#!/usr/bin/python

from setuptools import setup, find_packages

setup(name='hgspellcheck',
	version='1.0rc1',
	author='Iain Lowe',
	author_email='me@ilowe.net',
	description='Hook and extension to spellcheck Mercurial commit messages',
	long_description="""
To check all commits for spelling mistakes, and interactively edit them when
they occur, add the extension to the "extensions" section of your .hgrc.

  [extensions]
  hgspellcheck=

To reject all inbound commits that have spelling mistakes, add the hook
on "pretxnchangegroup".

  [hooks]
  # Reject all inbound changesets with spelling errors in their commit
  # messages
  pretxnchangegroup = python:hgspellcheck.hook
  
  # Refuse to create changesets in the local repository if their commit
  # messages contain spelling errors; this works for any commands that
  # create changesets (e.g. commit, qnew, qrecord)
  pretxncommit = python:hgspellcheck.hook
	""".strip(),
	py_modules=['hgspellcheck'],
	license='MIT',
	zip_safe=True,
	url='http://bitbucket.org/ilowe/hgspellcheck/downloads',
	keywords='mercurial hg scm spelling spellcheck spellchecker',
	install_requires = ['pyenchant>=1.5.3'],
)
